@@ -54,6 +54,38 @@ anjuta_plugin_description_parse_error_get_type (void)
   return etype;
 }
 
+/* enumerations from "anjuta-plugin-manager.h" */
+GType
+anjuta_plugin_manager_error_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { ANJUTA_PLUGIN_MANAGER_ERROR_INVALID_TYPE, "ANJUTA_PLUGIN_MANAGER_ERROR_INVALID_TYPE", "invalid-type" },
+      { ANJUTA_PLUGIN_MANAGER_ERROR_UNKNOWN, "ANJUTA_PLUGIN_MANAGER_ERROR_UNKNOWN", "unknown" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static (g_intern_static_string ("AnjutaPluginManagerError"), values);
+  }
+  return etype;
+}
+
+/* enumerations from "anjuta-profile.h" */
+GType
+anjuta_profile_error_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { ANJUTA_PROFILE_ERROR_URI_READ_FAILED, "ANJUTA_PROFILE_ERROR_URI_READ_FAILED", "read-failed" },
+      { ANJUTA_PROFILE_ERROR_URI_WRITE_FAILED, "ANJUTA_PROFILE_ERROR_URI_WRITE_FAILED", "write-failed" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static (g_intern_static_string ("AnjutaProfileError"), values);
+  }
+  return etype;
+}
+
 /* enumerations from "anjuta-preferences.h" */
 GType
 anjuta_property_object_type_get_type (void)
