@@ -18,6 +18,18 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/**
+ * SECTION:anjuta-plugin-handle
+ * @short_description: Handle used by plugin manager to keep track of plugins.
+ * @see_also: #AnjutaPluginManager, #AnjutaPluginDescription, #AnjutaPlugin
+ * @stability: Unstable
+ * @include: libanjuta/anjuta-plugin-handle.h
+ *
+ * Plugin handle is wrapper for installed plugins. For each installed plugin
+ * there will be one corresponding plugin handle. It is mainly used by
+ * plugin manager to keep track of loading and unloading of plugins.
+ */
+
 #include <string.h>
 #include "anjuta-plugin-handle.h"
 #include "resources.h"
@@ -480,8 +492,11 @@ anjuta_plugin_handle_new (const gchar *plugin_desc_path)
 		if (str && strcasecmp (str, "no") == 0)
 		{
 			plugin_handle->priv->user_activatable = FALSE;
+			/*
 			DEBUG_PRINT ("Plugin '%s' is not user activatable",
-						 plugin_handle->priv->name? plugin_handle->priv->name : "Unknown");
+						 plugin_handle->priv->name?
+						 plugin_handle->priv->name : "Unknown");
+			*/
 		}
 		g_free (str);
 	}
@@ -607,3 +622,25 @@ anjuta_plugin_handle_set_resolve_pass (AnjutaPluginHandle *plugin_handle,
 	g_return_if_fail (ANJUTA_IS_PLUGIN_HANDLE (plugin_handle));
 	plugin_handle->priv->resolve_pass = resolve_pass;
 }
+
+static gboolean
+g_hashtable_foreach_true (gpointer key, gpointer value, gpointer user_data)
+{
+	return TRUE;
+}
+
+void
+anjuta_plugin_handle_unresolve_dependencies (AnjutaPluginHandle *plugin_handle)
+{
+	AnjutaPluginHandlePriv *priv;
+	
+	g_return_if_fail (ANJUTA_IS_PLUGIN_HANDLE (plugin_handle));
+	priv = plugin_handle->priv;
+	
+	g_hash_table_foreach_remove (priv->dependencies, g_hashtable_foreach_true,
+								 NULL);
+	g_hash_table_foreach_remove (priv->dependents, g_hashtable_foreach_true,
+								 NULL);
+	priv->can_load = TRUE;
+	priv->resolve_pass = -1;
+}
