@@ -18,9 +18,19 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+/**
+ * SECTION:anjuta-plugin-manager
+ * @short_description: Plugins management and activation
+ * @see_also: #AnjutaPlugin, #AnjutaProfileManager
+ * @stability: Unstable
+ * @include: libanjuta/anjuta-plugin-manager.h
+ * 
+ */
+
 #include <sys/types.h>
 #include <dirent.h>
 #include <string.h>
+#include <libgnomevfs/gnome-vfs.h>
 
 #include <libanjuta/anjuta-plugin-manager.h>
 #include <libanjuta/anjuta-marshal.h>
@@ -57,7 +67,6 @@ struct _AnjutaPluginManagerPriv
 {
 	GObject      *shell;
 	AnjutaStatus *status;
-	GList        *profiles;
 	GList        *plugin_dirs;
 	GList        *available_plugins;
 	
@@ -71,6 +80,9 @@ struct _AnjutaPluginManagerPriv
 	
 	/* Plugins that have been previously loaded but current deactivated */
 	GHashTable   *plugins_cache;
+	
+	/* Remember plugin selection */
+	GHashTable   *remember_plugins;
 };
 
 enum {
@@ -91,8 +103,19 @@ static GObjectClass* parent_class = NULL;
 static guint plugin_manager_signals[LAST_SIGNAL] = { 0 };
 
 static GHashTable* plugin_set_update (AnjutaPluginManager *plugin_manager,
-									AnjutaPluginHandle* selected_plugin,
-									gboolean load);
+									  AnjutaPluginHandle* selected_plugin,
+									  gboolean load);
+
+GQuark 
+anjuta_plugin_manager_error_quark (void)
+{
+	static GQuark quark = 0;
+	
+	if (quark == 0) {
+		quark = g_quark_from_static_string ("anjuta-plugin-manager-quark");
+	}
+	return quark;
+}
 
 /** Dependency Resolution **/
 
@@ -116,7 +139,7 @@ collect_cycle (AnjutaPluginManager *plugin_manager,
 			if (dep == base_plugin)
 			{
 				*cycle = g_list_prepend (NULL, dep);
-				DEBUG_PRINT ("%s ", anjuta_plugin_handle_get_name (dep));
+				/* DEBUG_PRINT ("%s ", anjuta_plugin_handle_get_name (dep)); */
 				return TRUE;
 			}
 			else
@@ -124,7 +147,7 @@ collect_cycle (AnjutaPluginManager *plugin_manager,
 				if (collect_cycle (plugin_manager, base_plugin, dep, cycle))
 				{
 					*cycle = g_list_prepend (*cycle, dep);
-					DEBUG_PRINT ("%s ", anjuta_plugin_handle_get_name (dep));
+					/* DEBUG_PRINT ("%s ", anjuta_plugin_handle_get_name (dep)); */
 					return TRUE;
 				}
 			}
@@ -231,13 +254,10 @@ unresolve_dependencies (AnjutaPluginManager *plugin_manager)
 	
 	priv = plugin_manager->priv;
 	
-	for (l = priv->available_plugins; l != NULL; l = l->next) {
+	for (l = priv->available_plugins; l != NULL; l = l->next)
+	{
 		AnjutaPluginHandle *plugin = l->data;
-		
-		g_hash_table_remove_all (anjuta_plugin_handle_get_dependencies (plugin));
-		g_hash_table_remove_all (anjuta_plugin_handle_get_dependents (plugin));
-		anjuta_plugin_handle_set_can_load (plugin, TRUE);
-		anjuta_plugin_handle_set_resolve_pass (plugin, -1);
+		anjuta_plugin_handle_unresolve_dependencies (plugin);
 	}	
 }
 
@@ -461,6 +481,10 @@ on_plugin_activated (AnjutaPlugin *plugin_object, AnjutaPluginHandle *plugin)
 						 G_OBJECT (plugin_object));
 	if (g_hash_table_lookup (priv->plugins_cache, plugin))
 		g_hash_table_remove (priv->plugins_cache, plugin);
+	
+	g_signal_emit_by_name (plugin_manager, "plugin-activated",
+						   anjuta_plugin_handle_get_description (plugin),
+						   plugin_object);
 }
 
 static void
@@ -475,10 +499,15 @@ on_plugin_deactivated (AnjutaPlugin *plugin_object, AnjutaPluginHandle *plugin)
 	
 	g_hash_table_insert (priv->plugins_cache, plugin, G_OBJECT (plugin_object));
 	g_hash_table_remove (priv->activated_plugins, plugin);
+	
+	g_signal_emit_by_name (plugin_manager, "plugin-deactivated",
+						   anjuta_plugin_handle_get_description (plugin),
+						   plugin_object);
 }
 
 static GObject *
-activate_plugin (AnjutaPluginManager *plugin_manager, AnjutaPluginHandle *plugin)
+activate_plugin (AnjutaPluginManager *plugin_manager,
+				 AnjutaPluginHandle *plugin, GError **error)
 {
 	AnjutaPluginManagerPriv *priv;
 	GType type;
@@ -509,7 +538,9 @@ activate_plugin (AnjutaPluginManager *plugin_manager, AnjutaPluginHandle *plugin
 			g_strfreev (pieces);
 			if (ret == NULL)
 			{
-				g_warning ("Could not load C++ plugin: %s\n", plugin_id);
+				g_set_error (error, ANJUTA_PLUGIN_MANAGER_ERROR,
+							 ANJUTA_PLUGIN_MANAGER_ERROR_INVALID_TYPE,
+							 _("Invalid plugin: %s"), plugin_id);
 				return NULL;
 			}
 			g_object_set (ret, "shell", priv->shell, NULL);
@@ -526,7 +557,7 @@ activate_plugin (AnjutaPluginManager *plugin_manager, AnjutaPluginHandle *plugin
 	if (!type)
 	{
 		char **pieces;
-		DEBUG_PRINT ("Loading: %s", plugin_id);
+		/* DEBUG_PRINT ("Loading: %s", plugin_id); */
 		pieces = g_strsplit (plugin_id, ":", -1);
 		type = glue_factory_get_object_type (glue_factory,
 										     pieces[0], pieces[1]);
@@ -537,7 +568,9 @@ activate_plugin (AnjutaPluginManager *plugin_manager, AnjutaPluginHandle *plugin
 	
 	if (type == G_TYPE_INVALID)
 	{
-		g_warning ("Invalid type: Can not load %s\n", plugin_id);
+		g_set_error (error, ANJUTA_PLUGIN_MANAGER_ERROR,
+					 ANJUTA_PLUGIN_MANAGER_ERROR_INVALID_TYPE,
+					 _("Invalid plugin: %s"), plugin_id);
 		ret = NULL;
 	}
 	else
@@ -551,6 +584,12 @@ activate_plugin (AnjutaPluginManager *plugin_manager, AnjutaPluginHandle *plugin
 	return ret;
 }
 
+static gboolean
+g_hashtable_foreach_true (gpointer key, gpointer value, gpointer user_data)
+{
+	return TRUE;
+}
+
 void
 anjuta_plugin_manager_unload_all_plugins (AnjutaPluginManager *plugin_manager)
 {
@@ -571,12 +610,14 @@ anjuta_plugin_manager_unload_all_plugins (AnjutaPluginManager *plugin_manager)
 				if (g_hash_table_lookup (priv->activated_plugins, selected_plugin))
 				{
 					plugin_set_update (plugin_manager, selected_plugin, FALSE);
-					DEBUG_PRINT ("Unloading plugin: %s",
+					/* DEBUG_PRINT ("Unloading plugin: %s",
 								 anjuta_plugin_handle_get_id (selected_plugin));
+					*/
 				}
 				node = g_list_next (node);
 			}
-			g_hash_table_remove_all (priv->activated_plugins);
+			g_hash_table_foreach_remove (priv->activated_plugins,
+										 g_hashtable_foreach_true, NULL);
 		}
 		if (g_hash_table_size (priv->plugins_cache) > 0)
 		{
@@ -588,16 +629,18 @@ anjuta_plugin_manager_unload_all_plugins (AnjutaPluginManager *plugin_manager)
 				AnjutaPluginHandle *selected_plugin = node->data;
 				
 				plugin_obj = g_hash_table_lookup (priv->plugins_cache,
-												selected_plugin);
+												  selected_plugin);
 				if (plugin_obj)
 				{
-					DEBUG_PRINT ("Destroying plugin: %s",
+					/* DEBUG_PRINT ("Destroying plugin: %s",
 								 anjuta_plugin_handle_get_id (selected_plugin));
+					*/
 					g_object_unref (plugin_obj);
 				}
 				node = g_list_next (node);
 			}
-			g_hash_table_remove_all (priv->plugins_cache);
+			g_hash_table_foreach_remove (priv->plugins_cache,
+										 g_hashtable_foreach_true, NULL);
 		}
 		priv->available_plugins = g_list_reverse (priv->available_plugins);
 	}
@@ -730,7 +773,12 @@ plugin_set_update (AnjutaPluginManager *plugin_manager,
 				GObject *plugin_obj;
 				plugin_obj = g_hash_table_lookup (priv->plugins_cache, plugin);
 				if (!plugin_obj)
-					plugin_obj = activate_plugin (plugin_manager, plugin);
+				{
+					GError *error = NULL;
+					plugin_obj = activate_plugin (plugin_manager, plugin,
+												  &error);
+				}
+				
 				if (plugin_obj)
 				{
 					anjuta_plugin_activate (ANJUTA_PLUGIN (plugin_obj));
@@ -1386,8 +1434,8 @@ anjuta_plugin_manager_query (AnjutaPluginManager *plugin_manager,
 		if (satisfied)
 		{
 			selected_plugins = g_list_prepend (selected_plugins, desc);
-			DEBUG_PRINT ("Satisfied, Adding %s",
-						 anjuta_plugin_handle_get_name (plugin));
+			/* DEBUG_PRINT ("Satisfied, Adding %s",
+						 anjuta_plugin_handle_get_name (plugin));*/
 		}
 		available = g_list_next (available);
 	}
@@ -1419,7 +1467,23 @@ anjuta_plugin_manager_select (AnjutaPluginManager *plugin_manager,
 							  gchar *title, gchar *description,
 							  GList *plugin_descriptions)
 {
+	AnjutaPluginDescription *desc;
 	AnjutaPluginManagerPriv *priv;
+	GtkWidget *dlg;
+	GtkTreeModel *model;
+	GtkWidget *view;
+	GtkTreeViewColumn *column;
+	GtkCellRenderer *renderer;
+	GList *node;
+	GtkWidget *label;
+	GtkWidget *sc;
+	GtkWidget *remember_checkbox;
+	gint response;
+	GtkTreeIter selected;
+	GtkTreeSelection *selection;
+	GtkTreeModel *store;
+	GList *selection_ids = NULL;
+	GString *remember_key = g_string_new ("");
 	
 	g_return_val_if_fail (title != NULL, NULL);
 	g_return_val_if_fail (description != NULL, NULL);
@@ -1427,163 +1491,198 @@ anjuta_plugin_manager_select (AnjutaPluginManager *plugin_manager,
 
 	priv = plugin_manager->priv;
 	
-	if (g_list_length (plugin_descriptions) > 0)
+	if (g_list_length (plugin_descriptions) <= 0)
+		return NULL;
+		
+	dlg = gtk_dialog_new_with_buttons (title, GTK_WINDOW (priv->shell),
+									   GTK_DIALOG_DESTROY_WITH_PARENT,
+									   GTK_STOCK_CANCEL,
+									   GTK_RESPONSE_CANCEL,
+									   GTK_STOCK_OK, GTK_RESPONSE_OK,
+									   NULL);
+	gtk_widget_set_size_request (dlg, 400, 300);
+	gtk_window_set_default_size (GTK_WINDOW (dlg), 400, 300);
+
+	label = gtk_label_new (description);
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), label,
+						FALSE, FALSE, 5);
+	
+	sc = gtk_scrolled_window_new (NULL, NULL);
+	gtk_widget_show (sc);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sc),
+										 GTK_SHADOW_IN);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sc),
+									GTK_POLICY_AUTOMATIC,
+									GTK_POLICY_AUTOMATIC);
+	
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dlg)->vbox), sc,
+						TRUE, TRUE, 5);
+	
+	model = GTK_TREE_MODEL (gtk_list_store_new (N_COLUMNS, GDK_TYPE_PIXBUF,
+										   G_TYPE_STRING, G_TYPE_POINTER));
+	view = gtk_tree_view_new_with_model (model);
+	gtk_widget_show (view);
+	gtk_container_add (GTK_CONTAINER (sc), view);
+	
+	column = gtk_tree_view_column_new ();
+	gtk_tree_view_column_set_sizing (column,
+									 GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_column_set_title (column, _("Available Plugins"));
+
+	renderer = gtk_cell_renderer_pixbuf_new ();
+	gtk_tree_view_column_pack_start (column, renderer, FALSE);
+	gtk_tree_view_column_add_attribute (column, renderer, "pixbuf",
+										PIXBUF_COLUMN);
+
+	renderer = gtk_cell_renderer_text_new ();
+	gtk_tree_view_column_pack_start (column, renderer, TRUE);
+	gtk_tree_view_column_add_attribute (column, renderer, "markup",
+										PLUGIN_COLUMN);
+
+	gtk_tree_view_append_column (GTK_TREE_VIEW (view), column);
+	gtk_tree_view_set_expander_column (GTK_TREE_VIEW (view), column);
+	
+	g_signal_connect (view, "row-activated",
+					  G_CALLBACK (on_plugin_list_row_activated),
+					  GTK_DIALOG(dlg));
+	remember_checkbox =
+		gtk_check_button_new_with_label (_("Remember this selection"));
+	gtk_container_set_border_width (GTK_CONTAINER (remember_checkbox), 10);
+	gtk_widget_show (remember_checkbox);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dlg)->vbox), remember_checkbox,
+						FALSE, FALSE, 0);
+	
+	node = plugin_descriptions;
+	while (node)
 	{
-		GtkWidget *dlg;
-		GtkTreeModel *model;
-		GtkWidget *view;
-		GtkTreeViewColumn *column;
-		GtkCellRenderer *renderer;
-		GList *node;
-		GtkWidget *label;
-		GtkWidget *sc;
-		gint response;
+		GdkPixbuf *icon_pixbuf = NULL;
+		gchar *plugin_name = NULL;
+		gchar *plugin_desc = NULL;
+		gchar *icon_filename = NULL;
+		gchar *location = NULL;
 		
-		dlg = gtk_dialog_new_with_buttons (title, GTK_WINDOW (priv->shell),
-										   GTK_DIALOG_DESTROY_WITH_PARENT,
-										   GTK_STOCK_CANCEL,
-										   GTK_RESPONSE_CANCEL,
-										   GTK_STOCK_OK, GTK_RESPONSE_OK,
-										   NULL);
-		gtk_widget_set_size_request (dlg, 400, 300);
-		gtk_window_set_default_size (GTK_WINDOW (dlg), 400, 300);
-
-		label = gtk_label_new (description);
-		gtk_widget_show (label);
-		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), label,
-							FALSE, FALSE, 5);
+		desc = (AnjutaPluginDescription*)node->data;
 		
-		sc = gtk_scrolled_window_new (NULL, NULL);
-		gtk_widget_show (sc);
-		gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sc),
-											 GTK_SHADOW_IN);
-		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sc),
-										GTK_POLICY_AUTOMATIC,
-										GTK_POLICY_AUTOMATIC);
-		
-		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(dlg)->vbox), sc,
-							TRUE, TRUE, 5);
-		
-		model = GTK_TREE_MODEL (gtk_list_store_new (N_COLUMNS, GDK_TYPE_PIXBUF,
-											   G_TYPE_STRING, G_TYPE_POINTER));
-		view = gtk_tree_view_new_with_model (model);
-		gtk_widget_show (view);
-		gtk_container_add (GTK_CONTAINER (sc), view);
-		
-		column = gtk_tree_view_column_new ();
-		gtk_tree_view_column_set_sizing (column,
-										 GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-		gtk_tree_view_column_set_title (column, _("Available Plugins"));
-	
-		renderer = gtk_cell_renderer_pixbuf_new ();
-		gtk_tree_view_column_pack_start (column, renderer, FALSE);
-		gtk_tree_view_column_add_attribute (column, renderer, "pixbuf",
-											PIXBUF_COLUMN);
-	
-		renderer = gtk_cell_renderer_text_new ();
-		gtk_tree_view_column_pack_start (column, renderer, TRUE);
-		gtk_tree_view_column_add_attribute (column, renderer, "markup",
-											PLUGIN_COLUMN);
-	
-		gtk_tree_view_append_column (GTK_TREE_VIEW (view), column);
-		gtk_tree_view_set_expander_column (GTK_TREE_VIEW (view), column);
-		
-		g_signal_connect (view, "row-activated",
-						  G_CALLBACK (on_plugin_list_row_activated),
-						  GTK_DIALOG(dlg));
-
-		node = plugin_descriptions;
-		while (node)
+		if (anjuta_plugin_description_get_string (desc,
+												  "Anjuta Plugin",
+												  "Icon",
+												  &icon_filename))
 		{
-			GdkPixbuf *icon_pixbuf = NULL;
-			gchar *plugin_name = NULL;
-			gchar *plugin_desc = NULL;
-			gchar *icon_filename = NULL;
+			gchar *icon_path = NULL;
+			icon_path = g_strconcat (PACKAGE_PIXMAPS_DIR"/",
+									 icon_filename, NULL);
+			/* DEBUG_PRINT ("Icon: %s", icon_path); */
+			icon_pixbuf = 
+				gdk_pixbuf_new_from_file (icon_path, NULL);
+			if (icon_pixbuf == NULL)
+			{
+				g_warning ("Plugin pixmap not found: %s", plugin_name);
+			}
+			g_free (icon_path);
+		}
+		else
+		{
+			g_warning ("Plugin does not define Icon attribute");
+		}
+		if (!anjuta_plugin_description_get_string (desc,
+												  "Anjuta Plugin",
+												  "Name",
+												  &plugin_name))
+		{
+			g_warning ("Plugin does not define Name attribute");
+		}
+		if (!anjuta_plugin_description_get_string (desc,
+												  "Anjuta Plugin",
+												  "Description",
+												  &plugin_desc))
+		{
+			g_warning ("Plugin does not define Description attribute");
+		}
+		if (!anjuta_plugin_description_get_string (desc,
+												  "Anjuta Plugin",
+												  "Location",
+												  &location))
+		{
+			g_warning ("Plugin does not define Location attribute");
+		}
+		
+		if (plugin_name && plugin_desc)
+		{
+			GtkTreeIter iter;
+			gchar *text;
 			
-			AnjutaPluginDescription *desc =
-				(AnjutaPluginDescription*)node->data;
-			
-			if (anjuta_plugin_description_get_string (desc,
-													  "Anjuta Plugin",
-													  "Icon",
-													  &icon_filename))
-			{
-				gchar *icon_path = NULL;
-				icon_path = g_strconcat (PACKAGE_PIXMAPS_DIR"/",
-										 icon_filename, NULL);
-				DEBUG_PRINT ("Icon: %s", icon_path);
-				icon_pixbuf = 
-					gdk_pixbuf_new_from_file (icon_path, NULL);
-				if (icon_pixbuf == NULL)
-				{
-					g_warning ("Plugin pixmap not found: %s", plugin_name);
-				}
-				g_free (icon_path);
-			}
-			else
-			{
-				g_warning ("Plugin does not define Icon attribute");
-			}
-			if (!anjuta_plugin_description_get_string (desc,
-													  "Anjuta Plugin",
-													  "Name",
-													  &plugin_name))
-			{
-				g_warning ("Plugin does not define Name attribute");
-			}
-			if (!anjuta_plugin_description_get_string (desc,
-													  "Anjuta Plugin",
-													  "Description",
-													  &plugin_desc))
-			{
-				g_warning ("Plugin does not define Description attribute");
-			}
-			if (plugin_name && plugin_desc)
-			{
-				GtkTreeIter iter;
-				gchar *text;
-				
-				text = g_markup_printf_escaped ("<span size=\"larger\" weight=\"bold\">%s</span>\n%s", plugin_name, plugin_desc);
+			text = g_markup_printf_escaped ("<span size=\"larger\" weight=\"bold\">%s</span>\n%s", plugin_name, plugin_desc);
 
-				gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+			gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+								PLUGIN_COLUMN, text,
+								PLUGIN_DESCRIPTION_COLUMN, desc, -1);
+			if (icon_pixbuf) {
 				gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-									PLUGIN_COLUMN, text,
-									PLUGIN_DESCRIPTION_COLUMN, desc, -1);
-				if (icon_pixbuf) {
-					gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-										PIXBUF_COLUMN, icon_pixbuf, -1);
-					g_object_unref (icon_pixbuf);
-				}
-				g_free (text);
+									PIXBUF_COLUMN, icon_pixbuf, -1);
+				g_object_unref (icon_pixbuf);
 			}
-			node = g_list_next (node);
+			g_free (text);
+			
+			selection_ids = g_list_prepend (selection_ids, location);
 		}
-		response = gtk_dialog_run (GTK_DIALOG (dlg));
-		switch (response)
-		{
-		case GTK_RESPONSE_OK:
-			{
-				GtkTreeIter selected;
-				GtkTreeSelection *selection;
-				GtkTreeModel *store;
-				
-				selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
-				if (gtk_tree_selection_get_selected (selection, &store,
-													 &selected))
-				{
-					AnjutaPluginDescription *d;
-					gtk_tree_model_get (model, &selected,
-										PLUGIN_DESCRIPTION_COLUMN, &d, -1);
-					if (d)
-					{
-						gtk_widget_destroy (dlg);
-						return d;
-					}
-				}
-			}
-		}
-		gtk_widget_destroy (dlg);
+		node = g_list_next (node);
 	}
+	
+	/* Prepare remembering key */
+	selection_ids = g_list_sort (selection_ids,
+								 (GCompareFunc)strcmp);
+	node = selection_ids;
+	while (node)
+	{
+		g_string_append (remember_key, (gchar*)node->data);
+		g_string_append (remember_key, ",");
+		node = g_list_next (node);
+	}
+	g_list_foreach (selection_ids, (GFunc) g_free, NULL);
+	g_list_free (selection_ids);
+	
+	/* Find if the selection is remembered */
+	desc = g_hash_table_lookup (priv->remember_plugins, remember_key->str);
+	if (desc)
+	{
+		g_string_free (remember_key, TRUE);
+		gtk_widget_destroy (dlg);
+		return desc;
+	}
+	
+	/* Prompt dialog */
+	response = gtk_dialog_run (GTK_DIALOG (dlg));
+	switch (response)
+	{
+	case GTK_RESPONSE_OK:
+		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
+		if (gtk_tree_selection_get_selected (selection, &store,
+											 &selected))
+		{
+			gtk_tree_model_get (model, &selected,
+								PLUGIN_DESCRIPTION_COLUMN, &desc, -1);
+			if (desc)
+			{
+				/* Remember selection */
+				if (gtk_toggle_button_get_active
+					(GTK_TOGGLE_BUTTON (remember_checkbox)))
+				{
+					/* DEBUG_PRINT ("Remembering selection '%s'",
+								 remember_key->str);*/
+					g_hash_table_insert (priv->remember_plugins,
+										 g_strdup (remember_key->str), desc);
+				}
+				g_string_free (remember_key, TRUE);
+				gtk_widget_destroy (dlg);
+				return desc;
+			}
+		}
+		break;
+	}
+	g_string_free (remember_key, TRUE);
+	gtk_widget_destroy (dlg);
 	return NULL;
 }
 
@@ -1634,6 +1733,9 @@ anjuta_plugin_manager_init (AnjutaPluginManager *object)
 													  g_direct_equal);
 	object->priv->plugins_cache = g_hash_table_new (g_direct_hash,
 												  g_direct_equal);
+	object->priv->remember_plugins = g_hash_table_new_full (g_str_hash,
+															g_str_equal,
+															g_free, NULL);
 }
 
 static void
@@ -1641,12 +1743,6 @@ anjuta_plugin_manager_finalize (GObject *object)
 {
 	AnjutaPluginManagerPriv *priv;
 	priv = ANJUTA_PLUGIN_MANAGER (object)->priv;
-	if (priv->profiles)
-	{
-		g_list_foreach (priv->profiles, (GFunc)g_object_unref, NULL);
-		g_list_free (priv->profiles);
-		priv->profiles = NULL;
-	}
 	if (priv->available_plugins)
 	{
 		/* anjuta_plugin_manager_unload_all_plugins (ANJUTA_PLUGIN_MANAGER (object)); */
@@ -1676,7 +1772,6 @@ anjuta_plugin_manager_finalize (GObject *object)
 	}
 	if (priv->plugins_by_interfaces)
 	{
-		g_hash_table_remove_all (priv->plugins_by_interfaces);
 		g_hash_table_destroy (priv->plugins_by_interfaces);
 		priv->plugins_by_interfaces = NULL;
 	}
@@ -1718,17 +1813,6 @@ anjuta_plugin_manager_set_property (GObject *object, guint prop_id,
 	case PROP_SHELL:
 		priv->shell = g_value_get_object (value);
 		break;
-#if 0 /* Non setable properties */
-	case PROP_PROFILES:
-		/* TODO: Add setter for "profiles" property here */
-		break;
-	case PROP_AVAILABLE_PLUGINS:
-		/* TODO: Add setter for "available-plugins" property here */
-		break;
-	case PROP_ACTIVATED_PLUGINS:
-		/* TODO: Add setter for "activated-plugins" property here */
-		break;
-#endif
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -1752,9 +1836,6 @@ anjuta_plugin_manager_get_property (GObject *object, guint prop_id,
 	case PROP_STATUS:
 		g_value_set_object (value, priv->status);
 		break;
-	case PROP_PROFILES:
-		g_value_set_pointer (value, priv->profiles);
-		break;
 	case PROP_AVAILABLE_PLUGINS:
 		g_value_set_pointer (value, priv->available_plugins);
 		break;
@@ -1766,38 +1847,8 @@ anjuta_plugin_manager_get_property (GObject *object, guint prop_id,
 		break;
 	}
 }
-
-static void
-anjuta_plugin_manager_profile_pushed (AnjutaPluginManager *self,
-									  AnjutaProfile* profile)
-{
-	/* TODO: Add default signal handler implementation here */
-}
-
-static void
-anjuta_plugin_manager_profile_popped (AnjutaPluginManager *self,
-									  AnjutaProfile* profile)
-{
-	/* TODO: Add default signal handler implementation here */
-}
-
-static void
-anjuta_plugin_manager_plugins_to_load (AnjutaPluginManager *self,
-									   GList* plugins_to_load)
-{
-	/* TODO: Add default signal handler implementation here */
-}
-
-static void
-anjuta_plugin_manager_plugins_to_unload (AnjutaPluginManager *self,
-										 GList* plugins_to_unload)
-{
-	/* TODO: Add default signal handler implementation here */
-}
-
 static void
 anjuta_plugin_manager_plugin_activated (AnjutaPluginManager *self,
-										AnjutaProfile* profile,
 										AnjutaPluginDescription* plugin_desc,
 										GObject *plugin)
 {
@@ -1806,8 +1857,8 @@ anjuta_plugin_manager_plugin_activated (AnjutaPluginManager *self,
 
 static void
 anjuta_plugin_manager_plugin_deactivated (AnjutaPluginManager *self,
-										  AnjutaProfile* profile,
-										  AnjutaPluginDescription* plugin_desc)
+										  AnjutaPluginDescription* plugin_desc,
+										  GObject *plugin)
 {
 	/* TODO: Add default signal handler implementation here */
 }
@@ -1822,10 +1873,6 @@ anjuta_plugin_manager_class_init (AnjutaPluginManagerClass *klass)
 	object_class->set_property = anjuta_plugin_manager_set_property;
 	object_class->get_property = anjuta_plugin_manager_get_property;
 
-	klass->profile_pushed = anjuta_plugin_manager_profile_pushed;
-	klass->profile_popped = anjuta_plugin_manager_profile_popped;
-	klass->plugins_to_load = anjuta_plugin_manager_plugins_to_load;
-	klass->plugins_to_unload = anjuta_plugin_manager_plugins_to_unload;
 	klass->plugin_activated = anjuta_plugin_manager_plugin_activated;
 	klass->plugin_deactivated = anjuta_plugin_manager_plugin_deactivated;
 
@@ -1867,50 +1914,6 @@ anjuta_plugin_manager_class_init (AnjutaPluginManagerClass *klass)
 														  G_PARAM_WRITABLE |
 														  G_PARAM_CONSTRUCT));
 	
-	plugin_manager_signals[PROFILE_PUSHED] =
-		g_signal_new ("profile-pushed",
-		              G_OBJECT_CLASS_TYPE (klass),
-		              G_SIGNAL_RUN_FIRST,
-		              G_STRUCT_OFFSET (AnjutaPluginManagerClass,
-									   profile_pushed),
-		              NULL, NULL,
-		              anjuta_cclosure_marshal_VOID__OBJECT,
-		              G_TYPE_NONE, 1,
-		              ANJUTA_TYPE_PROFILE);
-
-	plugin_manager_signals[PROFILE_POPPED] =
-		g_signal_new ("profile-popped",
-		              G_OBJECT_CLASS_TYPE (klass),
-		              G_SIGNAL_RUN_FIRST,
-		              G_STRUCT_OFFSET (AnjutaPluginManagerClass,
-									   profile_popped),
-		              NULL, NULL,
-					  anjuta_cclosure_marshal_VOID__OBJECT,
-		              G_TYPE_NONE, 1,
-		              ANJUTA_TYPE_PROFILE);
-
-	plugin_manager_signals[PLUGINS_TO_LOAD] =
-		g_signal_new ("plugins-to-load",
-		              G_OBJECT_CLASS_TYPE (klass),
-		              G_SIGNAL_RUN_FIRST,
-		              G_STRUCT_OFFSET (AnjutaPluginManagerClass,
-									   plugins_to_load),
-		              NULL, NULL,
-					  anjuta_cclosure_marshal_VOID__POINTER,
-		              G_TYPE_NONE, 1,
-		              G_TYPE_POINTER);
-
-	plugin_manager_signals[PLUGINS_TO_UNLOAD] =
-		g_signal_new ("plugins-to-unload",
-		              G_OBJECT_CLASS_TYPE (klass),
-		              G_SIGNAL_RUN_FIRST,
-		              G_STRUCT_OFFSET (AnjutaPluginManagerClass,
-									   plugins_to_unload),
-		              NULL, NULL,
-					  anjuta_cclosure_marshal_VOID__POINTER,
-		              G_TYPE_NONE, 1,
-		              G_TYPE_POINTER);
-
 	plugin_manager_signals[PLUGIN_ACTIVATED] =
 		g_signal_new ("plugin-activated",
 		              G_OBJECT_CLASS_TYPE (klass),
@@ -1918,9 +1921,9 @@ anjuta_plugin_manager_class_init (AnjutaPluginManagerClass *klass)
 		              G_STRUCT_OFFSET (AnjutaPluginManagerClass,
 									   plugin_activated),
 		              NULL, NULL,
-					  anjuta_cclosure_marshal_VOID__OBJECT_POINTER_OBJECT,
-		              G_TYPE_NONE, 3,
-		              ANJUTA_TYPE_PROFILE, G_TYPE_POINTER, ANJUTA_TYPE_PLUGIN);
+					  anjuta_cclosure_marshal_VOID__POINTER_OBJECT,
+		              G_TYPE_NONE, 2,
+		              G_TYPE_POINTER, ANJUTA_TYPE_PLUGIN);
 
 	plugin_manager_signals[PLUGIN_DEACTIVATED] =
 		g_signal_new ("plugin-deactivated",
@@ -1929,9 +1932,9 @@ anjuta_plugin_manager_class_init (AnjutaPluginManagerClass *klass)
 		              G_STRUCT_OFFSET (AnjutaPluginManagerClass,
 									   plugin_deactivated),
 		              NULL, NULL,
-					  anjuta_cclosure_marshal_VOID__OBJECT_POINTER,
+					  anjuta_cclosure_marshal_VOID__POINTER_OBJECT,
 		              G_TYPE_NONE, 2,
-		              ANJUTA_TYPE_PROFILE, G_TYPE_POINTER);
+		              G_TYPE_POINTER, ANJUTA_TYPE_PLUGIN);
 }
 
 GType
@@ -2020,15 +2023,155 @@ anjuta_plugin_manager_new (GObject *shell, AnjutaStatus *status,
 }
 
 void
-anjuta_plugin_manager_push_profile (const gchar *profile_name,
-									const gchar* profile_xml_file,
-									gboolean readonly)
+anjuta_plugin_manager_activate_plugins (AnjutaPluginManager *plugin_manager,
+										GList *plugins_to_activate)
 {
-	/* TODO: Add public function implementation here */
+	AnjutaPluginManagerPriv *priv;
+	GdkPixbuf *icon_pixbuf;
+	GList *node;
+	
+	priv = plugin_manager->priv;
+	
+	/* Freeze shell operations */
+	anjuta_shell_freeze (ANJUTA_SHELL (priv->shell), NULL);
+	if (plugins_to_activate)
+	{
+		anjuta_status_progress_add_ticks (ANJUTA_STATUS (priv->status),
+										  g_list_length (plugins_to_activate));
+	}
+	node = plugins_to_activate;
+	while (node)
+	{
+		AnjutaPluginDescription *d;
+		gchar *plugin_id;
+		gchar *icon_filename, *label;
+		gchar *icon_path = NULL;
+		
+		d = node->data;
+		
+		icon_pixbuf = NULL;
+		label = NULL;
+		if (anjuta_plugin_description_get_string (d, "Anjuta Plugin",
+												  "Icon",
+											  &icon_filename))
+		{
+			gchar *title, *description;
+			anjuta_plugin_description_get_string (d, "Anjuta Plugin",
+												  "Name",
+												  &title);
+			anjuta_plugin_description_get_string (d, "Anjuta Plugin",
+												  "Description",
+												  &description);
+			icon_path = g_strconcat (PACKAGE_PIXMAPS_DIR"/",
+									 icon_filename, NULL);
+			/* DEBUG_PRINT ("Icon: %s", icon_path); */
+			label = g_strconcat (_("Loaded: "), title, _("..."), NULL);
+			icon_pixbuf = gdk_pixbuf_new_from_file (icon_path, NULL);
+			if (!icon_pixbuf)
+				g_warning ("Plugin does not define Icon: No such file %s",
+						   icon_path);
+			g_free (icon_path);
+			g_free (icon_filename);
+		}
+		
+		if (anjuta_plugin_description_get_string (d, "Anjuta Plugin",
+												  "Location", &plugin_id))
+		{
+			GObject *plugin_obj;
+			
+			plugin_obj =
+				anjuta_plugin_manager_get_plugin_by_id (plugin_manager,
+														plugin_id);
+			g_free (plugin_id);
+		}
+		anjuta_status_progress_tick (ANJUTA_STATUS (priv->status),
+									 icon_pixbuf, label);
+		g_free (label);
+		if (icon_pixbuf)
+			g_object_unref (icon_pixbuf);
+		
+		node = g_list_next (node);
+	}
+	
+	/* Thaw shell operations */
+	anjuta_shell_thaw (ANJUTA_SHELL (priv->shell), NULL);
+}
+
+static void
+on_collect (gpointer key, gpointer value, gpointer user_data)
+{
+	gchar *id;
+	gchar *query = (gchar*) key;
+	AnjutaPluginDescription *desc = (AnjutaPluginDescription *) value;
+	GString *write_buffer = (GString *) user_data;
+	
+	anjuta_plugin_description_get_string (desc, "Anjuta Plugin", "Location",
+										  &id);
+	g_string_append_printf (write_buffer, "%s=%s;", query, id);
+	g_free (id);
+}
+
+gchar*
+anjuta_plugin_manager_get_remembered_plugins (AnjutaPluginManager *plugin_manager)
+{
+	AnjutaPluginManagerPriv *priv;
+	GString *write_buffer = g_string_new ("");
+	
+	g_return_val_if_fail (ANJUTA_IS_PLUGIN_MANAGER (plugin_manager), FALSE);
+	
+	priv = plugin_manager->priv;
+	g_hash_table_foreach (priv->remember_plugins, on_collect,
+						  write_buffer);
+	return g_string_free (write_buffer, FALSE);
+}
+
+static gboolean
+on_foreach_remove_true (gpointer k, gpointer v, gpointer d)
+{
+	return TRUE;
 }
 
 void
-anjuta_plugin_manager_pop_profile (const gchar *profile_name)
+anjuta_plugin_manager_set_remembered_plugins (AnjutaPluginManager *plugin_manager,
+											  const gchar *remembered_plugins)
 {
-	/* TODO: Add public function implementation here */
+	AnjutaPluginManagerPriv *priv;
+	gchar **strv_lines, **line_idx;
+	
+	g_return_if_fail (ANJUTA_IS_PLUGIN_MANAGER (plugin_manager));
+	g_return_if_fail (remembered_plugins != NULL);
+	
+	priv = plugin_manager->priv;
+	
+	g_hash_table_foreach_remove (priv->remember_plugins,
+								 on_foreach_remove_true, NULL);
+	
+	strv_lines = g_strsplit (remembered_plugins, ";", -1);
+	line_idx = strv_lines;
+	while (*line_idx)
+	{
+		gchar **strv_keyvals;
+		strv_keyvals = g_strsplit (*line_idx, "=", -1);
+		if (strv_keyvals && strv_keyvals[0] && strv_keyvals[1])
+		{
+			AnjutaPluginHandle *plugin;
+			plugin = g_hash_table_lookup (priv->plugins_by_name,
+										  strv_keyvals[1]);
+			if (plugin)
+			{
+				AnjutaPluginDescription *desc;
+				desc = anjuta_plugin_handle_get_description (plugin);
+				/*
+				DEBUG_PRINT ("Restoring remember plugin: %s=%s",
+							 strv_keyvals[0],
+							 strv_keyvals[1]);
+				*/
+				g_hash_table_insert (priv->remember_plugins,
+									 g_strdup (strv_keyvals[0]), desc);
+			}
+			g_strfreev (strv_keyvals);
+		}
+		line_idx++;
+	}
+	g_strfreev (strv_lines);
 }
