@@ -17,6 +17,46 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+
+/**
+ * SECTION:anjuta-preferences
+ * @short_description: Anjuta Prefereces system.
+ * @see_also: #AnjutaPreferencesDialog
+ * @stability: Unstable
+ * @include: libanjuta/anjuta-preferences.h
+ * 
+ * #AnjutaPreferences is a way to let plugins register their preferences. There
+ * are mainly two ways a plugin could register its preferences in Anjuta.
+ * 
+ * First is to not use #AnjutaPreferences at all. Simply register a
+ * preferences page in #AnjutaPreferencesDialog using the function
+ * anjuta_preferences_dialog_add_page(). The plugin should take
+ * care of loading, saving and widgets synchronization of the
+ * preferences values. It is particularly useful if the plugin
+ * uses gconf system for its preferences. Also no "changed"
+ * signal will be emitted from it.
+ * 
+ * Second is to use anjuta_preferences_add_page(), which will
+ * automatically register the preferences keys and values from
+ * a glade xml file. The glade xml file contains a preferences
+ * page of the plugin. The widget names in the page are
+ * given in a particular way (see anjuta_preferences_add_page()) to
+ * let it know property key details. Loading, saving and
+ * widget synchronization are automatically done. "changed" signal is
+ * emitted when a preference is changed.
+ * 
+ * anjuta_preferences_register_all_properties_from_glade_xml() only registers
+ * the preferences propery keys for automatic loading, saving and widget
+ * syncrhronization, but does not add the page in preferences dialog. It
+ * is useful if the plugin wants to show the preferences page somewhere else.
+ * 
+ * anjuta_preferences_register_property_from_string() is similar to 
+ * anjuta_preferences_register_all_properties_from_glade_xml(), but it only
+ * registers one property, the detail of which is given in its arguments.
+ * anjuta_preferences_register_property_custom() is used to register a
+ * property that uses a widget which is not supported by #AnjutaPreferences.
+ */
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -660,7 +700,7 @@ set_property_value_as_string (AnjutaProperty *prop, const gchar *value)
 				font_name = g_strconcat (tmp, " 12", NULL);
 				g_free (tmp);
 				
-				DEBUG_PRINT ("Font set as: %s", font_name);
+				/* DEBUG_PRINT ("Font set as: %s", font_name); */
 				
 				gtk_font_button_set_font_name (GTK_FONT_BUTTON
 												 (prop->object), font_name);
@@ -798,7 +838,7 @@ get_property (GConfClient *gclient, guint cnxt_id,
 	AnjutaProperty *p = (AnjutaProperty *) user_data;
 	key = gconf_entry_get_key (entry);
 	value = gconf_entry_get_value (entry);
-	DEBUG_PRINT ("Preference changed %s", key);
+	/* DEBUG_PRINT ("Preference changed %s", key); */
 
 	if (p->data_type == ANJUTA_PROPERTY_DATA_TYPE_BOOL)
 	{
@@ -953,8 +993,8 @@ anjuta_preferences_register_property_raw (AnjutaPreferences *pr,
 				old_value = anjuta_preferences_get (pr, p->key);
 				if (old_value == NULL && vstr[0])
 				{
-					DEBUG_PRINT ("Setting default pref value: %s = %s",
-								 p->key, default_value);
+					/* DEBUG_PRINT ("Setting default pref value: %s = %s",
+								 p->key, default_value); */
 					anjuta_preferences_set (pr, p->key, vstr[0]);
 				}
 				if (old_value)
@@ -967,8 +1007,8 @@ anjuta_preferences_register_property_raw (AnjutaPreferences *pr,
 				old_value = anjuta_preferences_get (pr, p->key);
 				if (old_value == NULL)
 				{
-					DEBUG_PRINT ("Setting default pref value: %s = %s",
-								 p->key, default_value);
+					/* DEBUG_PRINT ("Setting default pref value: %s = %s",
+								 p->key, default_value);*/
 					anjuta_preferences_set (pr, p->key, default_value);
 				}
 				if (old_value)
@@ -980,8 +1020,8 @@ anjuta_preferences_register_property_raw (AnjutaPreferences *pr,
 										  build_key (p->key), NULL);
 				if (value == NULL)
 				{
-					DEBUG_PRINT ("Setting default pref value: %s = %s",
-								 p->key, default_value);
+					/* DEBUG_PRINT ("Setting default pref value: %s = %s",
+								 p->key, default_value);*/
 					if (p->data_type == ANJUTA_PROPERTY_DATA_TYPE_INT)
 						gconf_client_set_int (pr->priv->gclient,
 											  build_key (p->key),
@@ -1074,8 +1114,8 @@ anjuta_preferences_register_property_custom (AnjutaPreferences *pr,
 			old_value = anjuta_preferences_get (pr, p->key);
 			if (old_value == NULL)
 			{
-				DEBUG_PRINT ("Setting default pref value: %s = %s",
-							 p->key, default_value);
+				/* DEBUG_PRINT ("Setting default pref value: %s = %s",
+							 p->key, default_value); */
 				anjuta_preferences_set (pr, p->key, default_value);
 			}
 			if (old_value)
@@ -1087,8 +1127,8 @@ anjuta_preferences_register_property_custom (AnjutaPreferences *pr,
 									  build_key (p->key), NULL);
 			if (value == NULL)
 			{
-				DEBUG_PRINT ("Setting default pref value: %s = %s",
-							 p->key, default_value);
+				/* DEBUG_PRINT ("Setting default pref value: %s = %s",
+							 p->key, default_value);*/
 				if (p->data_type == ANJUTA_PROPERTY_DATA_TYPE_INT)
 					gconf_client_set_int (pr->priv->gclient,
 										  build_key (p->key),
@@ -1468,7 +1508,7 @@ anjuta_preferences_instance_init (AnjutaPreferences *pr)
 	pr->priv = g_new0(AnjutaPreferencesPriv, 1);
 	pr->priv->properties = NULL;
 	
-	DEBUG_PRINT ("Initializing AP Instance");
+	/* DEBUG_PRINT ("Initializing AP Instance"); */
 	
 	pr->priv->is_showing = FALSE;
 	pr->priv->gclient = gconf_client_get_default();
