@@ -17,6 +17,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+
+/**
+ * SECTION:anjuta-utils
+ * @title: Utilities
+ * @short_description: Utility functions
+ * @see_also: 
+ * @stability: Unstable
+ * @include: libanjuta/anjuta-utils.h
+ * 
+ */
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -931,6 +942,66 @@ gboolean anjuta_util_diff(const gchar* uri, const gchar* text)
 	}
 }
 
+gboolean
+anjuta_util_path_has_extension (const gchar *path, const gchar *ext)
+{
+	if (strlen (path) <= strlen (ext))
+		return FALSE;
+	if ((path[strlen (path) - strlen (ext) - 1] == '.') &&
+		(strcmp (&path[strlen (path) - strlen (ext)], ext) == 0))
+		return TRUE;
+	return FALSE;
+}
+
+gchar *
+anjuta_util_get_uri_mime_type (const gchar *uri)
+{
+	GnomeVFSURI *vfs_uri;
+	const gchar *path;
+	gchar *mime_type;
+	
+	g_return_val_if_fail (uri != NULL, NULL);
+	
+	vfs_uri = gnome_vfs_uri_new (uri);
+	if (vfs_uri)
+		path = gnome_vfs_uri_get_path (vfs_uri);
+	else
+		path = NULL;
+	
+	/* If Anjuta is not installed in system gnome prefix, the mime types 
+	 * may not have been correctly registed. In that case, we use the
+	 * following mime detection
+	 */
+	if (!path)
+	{
+		mime_type = gnome_vfs_get_mime_type (uri);
+	}
+	else if (anjuta_util_path_has_extension (path, "anjuta"))
+	{
+		mime_type = g_strdup ("application/x-anjuta");
+	}
+	else if (anjuta_util_path_has_extension (path, "prj"))
+	{
+		mime_type = g_strdup ("application/x-anjuta-old");
+	}
+	else if (anjuta_util_path_has_extension (path, "ui"))
+	{
+		mime_type = g_strdup ("text/xml");
+	}
+	else if (anjuta_util_path_has_extension (path, "glade"))
+	{
+		mime_type = g_strdup ("application/x-glade");
+	}
+	else
+	{
+		mime_type = gnome_vfs_get_mime_type (uri);
+	}
+	
+	if (vfs_uri)
+		gnome_vfs_uri_unref (vfs_uri);
+	return mime_type;
+}
+
 /*
  * Sun specific implementations
  *
@@ -1231,4 +1302,5 @@ int alphasort (const struct dirent **a, const struct dirent **b)
 {
     return (strcmp((*a)->d_name, (*b)->d_name));
 }
+
 #endif /* sun */
