@@ -1,3 +1,4 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
     breakpoints.c
     Copyright (C) 2000  Naba Kumar <naba@gnome.org>
@@ -41,7 +42,7 @@
 #  include <config.h>
 #endif
 
-//#define DEBUG
+/*#define DEBUG*/
 
 #include <sys/stat.h>
 #include <unistd.h>
@@ -58,7 +59,6 @@
 #include <libanjuta/interfaces/ianjuta-file.h>
 
 #include "breakpoints.h"
-#include "plugin.h"
 #include "utilities.h"
 
 /* Markers definition */
@@ -295,6 +295,7 @@ breakpoint_item_new_from_uri (BreakpointsDBase *bd, const gchar* uri, guint line
 	bi = g_new0 (BreakpointItem, 1);
 	bi->bd = bd;
 	bi->bp = anjuta_breakpoint_new();
+	bi->bp->type = IANJUTA_DEBUGGER_BREAK_ON_LINE;
 	if (uri != NULL)
 	{
 		bi->uri = g_strdup (uri);
@@ -492,7 +493,10 @@ breakpoint_item_update_in_ui (BreakpointItem *bi, const IAnjutaDebuggerBreakpoin
 	{
 			set_breakpoint_in_editor (bi, bi->bp->enable == IANJUTA_DEBUGGER_YES ? BREAKPOINT_ENABLED : BREAKPOINT_DISABLED, FALSE);
 	}
-	
+
+	/* Emit signal */
+	g_signal_emit_by_name (bi->bd->plugin, "breakpoint-changed", bi->bp);
+
 }
 
 static void
@@ -523,7 +527,11 @@ breakpoint_item_remove_in_ui (BreakpointItem *bi, const IAnjutaDebuggerBreakpoin
 
 	/* Delete breakpoint marker from screen */
 	set_breakpoint_in_editor (bi, BREAKPOINT_NONE, FALSE);
-		
+
+	/* Emit signal */
+	bi->bp->type = IANJUTA_DEBUGGER_BREAK_REMOVED;
+	g_signal_emit_by_name (bi->bd->plugin, "breakpoint-changed", bi->bp);
+	
 	breakpoint_item_free (bi);
 }
 
@@ -584,6 +592,9 @@ breakpoints_dbase_add_breakpoint (BreakpointsDBase *bd,  BreakpointItem *bi)
 					bi,
 					NULL);
 		    break;
+		default:
+			g_return_if_reached();
+			break;
 		}
 	}
 	else
@@ -1036,8 +1047,6 @@ breakpoints_dbase_set_all_in_editor (BreakpointsDBase* bd, IAnjutaEditor* te)
 	GtkTreeIter iter;
 	gchar *uri;
 	
-	DEBUG_PRINT("DMA: set_all_in_editor %p", te);
-	
 	g_return_if_fail (te != NULL);
 	g_return_if_fail (bd != NULL);
 	g_return_if_fail (bd->treeview != NULL);
@@ -1079,8 +1088,6 @@ breakpoints_dbase_clear_all_in_editor (BreakpointsDBase* bd, IAnjutaEditor* te)
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 
-	DEBUG_PRINT("DMA: clear_all_in_editor %p", te);
-	
 	g_return_if_fail (te != NULL);
 	g_return_if_fail (bd != NULL);
 	g_return_if_fail (bd->treeview != NULL);
@@ -1151,8 +1158,8 @@ on_jump_to_breakpoint_activate (GtkAction * action, BreakpointsDBase *bd)
 		BreakpointItem *bi;
 		
 		gtk_tree_model_get (model, &iter, DATA_COLUMN, &bi, -1);
-		
-		goto_location_in_editor (ANJUTA_PLUGIN (bd->plugin), bi->uri, bi->bp->line);
+
+		dma_debug_manager_goto_code (bd->plugin, bi->uri, bi->bp->line, bi->bp->address);
 	}
 }
 
@@ -1161,6 +1168,7 @@ on_breakpoints_button_press (GtkWidget * widget, GdkEventButton * bevent, Breakp
 {
 	if (bevent->button == 3)
 	{
+		g_return_val_if_fail (bd->popup != NULL, FALSE);
 		gtk_menu_popup (bd->popup, NULL, NULL, NULL, NULL,
 						bevent->button, bevent->time);
 	}
@@ -1174,19 +1182,10 @@ on_breakpoints_button_press (GtkWidget * widget, GdkEventButton * bevent, Breakp
 }
 
 static void
-on_treeview_enabled_toggled (GtkCellRendererToggle *cell,
-							 gchar			       *path_str,
-							 BreakpointsDBase      *bd)
+breakpoint_enable_disable (GtkTreeModel *model, GtkTreeIter iter, BreakpointsDBase *bd)
 {
 	BreakpointItem *bi;
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-	GtkTreePath *path;
 	
-	path = gtk_tree_path_new_from_string (path_str);
-
-	model = gtk_tree_view_get_model (bd->treeview);
-	gtk_tree_model_get_iter (model, &iter, path);
 	gtk_tree_model_get (model, &iter, DATA_COLUMN, &bi, -1);
 
 	if (bi->bp->enable == IANJUTA_DEBUGGER_YES)
@@ -1217,9 +1216,28 @@ on_treeview_enabled_toggled (GtkCellRendererToggle *cell,
 		gtk_list_store_set (GTK_LIST_STORE (model), &bi->iter,
 						ENABLED_COLUMN, bi->bp->enable == IANJUTA_DEBUGGER_YES ? TRUE : FALSE,
 						-1);
+		set_breakpoint_in_editor (bi, bi->bp->enable == IANJUTA_DEBUGGER_YES ? 
+								  BREAKPOINT_ENABLED : BREAKPOINT_DISABLED, TRUE);	
 	}
 }
 
+static void
+on_treeview_enabled_toggled (GtkCellRendererToggle *cell,
+							 gchar			       *path_str,
+							 BreakpointsDBase      *bd)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	GtkTreePath *path;
+	
+	path = gtk_tree_path_new_from_string (path_str);
+
+	model = gtk_tree_view_get_model (bd->treeview);
+	gtk_tree_model_get_iter (model, &iter, path);
+	
+	breakpoint_enable_disable (model, iter, bd);
+}
+
 static void
 on_clear_all_breakpoints (GtkWidget *button, BreakpointsDBase *bd)
 {
@@ -1268,7 +1286,7 @@ on_toggle_breakpoint_activate (GtkAction * action, BreakpointsDBase *bd)
 	if (bi == NULL)
 	{
 		bi = breakpoint_item_new_from_uri (bd, uri, line, TRUE);
-		
+	
 		breakpoints_dbase_add_breakpoint (bd, bi);
 	}
 	else
@@ -1333,12 +1351,28 @@ on_edit_breakpoint_activate (GtkAction * action, BreakpointsDBase *bd)
 	}
 }
 
+static void
+on_enable_breakpoint_activate (GtkAction * action, BreakpointsDBase *bd)
+{
+	GtkTreeModel *model;
+	GtkTreeSelection *selection;
+	GtkTreeIter iter;
+	gboolean valid;
+
+	selection =	gtk_tree_view_get_selection (bd->treeview);
+	valid = gtk_tree_selection_get_selected (selection, &model, &iter);
+	if (valid)
+	{
+		breakpoint_enable_disable (model, iter, bd);
+	}
+}
+
 /* Actions table
  *---------------------------------------------------------------------------*/
 
 static GtkActionEntry actions_breakpoints[] = {
 	{
-		"ActionMenuGdbBreakpoints",               /* Action name */
+		"ActionMenuDmaBreakpoints",               /* Action name */
 		NULL,                                     /* Stock icon, if any */
 		N_("_Breakpoints"),                       /* Display label */
 		NULL,                                     /* short-cut */
@@ -1346,7 +1380,7 @@ static GtkActionEntry actions_breakpoints[] = {
 		NULL                                      /* action callback */
 	},
 	{
-		"ActionGdbToggleBreakpoint",              /* Action name */
+		"ActionDmaToggleBreakpoint",              /* Action name */
 		"gdb-breakpoint-toggle",                  /* Stock icon, if any */
 		N_("Toggle Breakpoint"),                  /* Display label */
 		"<control>b",                             /* short-cut */
@@ -1354,7 +1388,7 @@ static GtkActionEntry actions_breakpoints[] = {
 		G_CALLBACK (on_toggle_breakpoint_activate) /* action callback */
 	},
 	{
-		"ActionGdbSetBreakpoint",                 /* Action name */
+		"ActionDmaSetBreakpoint",                 /* Action name */
 		"gdb-breakpoint-toggle",                  /* Stock icon, if any */
 		N_("Add Breakpoint..."),                  /* Display label */
 		NULL,                                     /* short-cut */
@@ -1362,7 +1396,7 @@ static GtkActionEntry actions_breakpoints[] = {
 		G_CALLBACK (on_add_breakpoint_activate)   /* action callback */
 	},
 	{
-		"ActionGdbClearBreakpoint",               /* Action name */
+		"ActionDmaClearBreakpoint",               /* Action name */
 		NULL,                                     /* Stock icon, if any */
 		N_("Remove Breakpoint"),                  /* Display label */
 		NULL,                                     /* short-cut */
@@ -1370,7 +1404,7 @@ static GtkActionEntry actions_breakpoints[] = {
 		G_CALLBACK (on_remove_breakpoint_activate)   /* action callback */
 	},
 	{
-		"ActionGdbJumpToBreakpoint",              /* Action name */
+		"ActionDmaJumpToBreakpoint",              /* Action name */
 		NULL,                                     /* Stock icon, if any */
 		N_("Jump to Breakpoint"),                 /* Display label */
 		NULL,                                     /* short-cut */
@@ -1378,7 +1412,7 @@ static GtkActionEntry actions_breakpoints[] = {
 		G_CALLBACK (on_jump_to_breakpoint_activate)   /* action callback */
 	},
 	{
-		"ActionGdbEditBreakpoint",                /* Action name */
+		"ActionDmaEditBreakpoint",                /* Action name */
 		NULL,                                     /* Stock icon, if any */
 		N_("Edit Breakpoint"),                    /* Display label */
 		NULL,                                     /* short-cut */
@@ -1386,15 +1420,15 @@ static GtkActionEntry actions_breakpoints[] = {
 		G_CALLBACK (on_edit_breakpoint_activate)   /* action callback */
 	},
 	{
-		"ActionGdbEnableDisableBreakpoint",       /* Action name */
+		"ActionDmaEnableDisableBreakpoint",       /* Action name */
 		NULL,                                     /* Stock icon, if any */
 		N_("Enable Breakpoint"),                  /* Display label */
 		NULL,                                     /* short-cut */
 		N_("Enable a breakpoint"),                /* Tooltip */
-		G_CALLBACK (on_add_breakpoint_activate)   /* action callback */
+		G_CALLBACK (on_enable_breakpoint_activate)   /* action callback */
 	},
 	{
-		"ActionGdbDisableAllBreakpoints",         /* Action name */
+		"ActionDmaDisableAllBreakpoints",         /* Action name */
 		NULL,                                     /* Stock icon, if any */
 		N_("Disable All Breakpoints"),            /* Display label */
 		NULL,                                     /* short-cut */
@@ -1402,7 +1436,7 @@ static GtkActionEntry actions_breakpoints[] = {
 		G_CALLBACK (on_disable_all_breakpoints_activate)/* action callback */
 	},
 	{
-		"ActionGdbClearAllBreakpoints",           /* Action name */
+		"ActionDmaClearAllBreakpoints",           /* Action name */
 		NULL,                                     /* Stock icon, if any */
 		N_("C_lear All Breakpoints"),             /* Display label */
 		NULL,                                     /* short-cut */
@@ -1446,7 +1480,7 @@ on_session_load (AnjutaShell *shell, AnjutaSessionPhase phase, AnjutaSession *se
  *---------------------------------------------------------------------------*/
 
 BreakpointsDBase *
-breakpoints_dbase_new (AnjutaPlugin *plugin)
+breakpoints_dbase_new (DebugManagerPlugin *plugin)
 {
 	BreakpointsDBase *bd;
 	AnjutaUI *ui;
@@ -1463,9 +1497,9 @@ breakpoints_dbase_new (AnjutaPlugin *plugin)
 
 
 		/* Connect to Load and Save event */
-		g_signal_connect (plugin->shell, "save-session",
+		g_signal_connect (ANJUTA_PLUGIN(plugin)->shell, "save-session",
 					  G_CALLBACK (on_session_save), bd);
-		g_signal_connect (plugin->shell, "load-session",
+		g_signal_connect (ANJUTA_PLUGIN(plugin)->shell, "load-session",
 					  G_CALLBACK (on_session_load), bd);
 		
 		/* breakpoints dialog */
@@ -1511,7 +1545,7 @@ breakpoints_dbase_new (AnjutaPlugin *plugin)
 		}
 
 		/* Register menu actions */
-		ui = anjuta_shell_get_ui (plugin->shell, NULL);
+		ui = anjuta_shell_get_ui (ANJUTA_PLUGIN(plugin)->shell, NULL);
 		bd->action_group =
 		anjuta_ui_add_action_group_entries (ui, "ActionGroupBreakpoint",
 											_("Breakpoint operations"),
@@ -1530,7 +1564,7 @@ breakpoints_dbase_new (AnjutaPlugin *plugin)
 		gtk_container_add (GTK_CONTAINER (bd->scrolledwindow), GTK_WIDGET (bd->treeview));
 		gtk_widget_show_all (bd->scrolledwindow);
 		
-		anjuta_shell_add_widget (plugin->shell,
+		anjuta_shell_add_widget (ANJUTA_PLUGIN(plugin)->shell,
 							 bd->scrolledwindow,
                              "AnjutaDebuggerBreakpoints", _("Breakpoints"),
                              "gdb-breakpoint-toggle", ANJUTA_SHELL_PLACEMENT_BOTTOM,
@@ -1543,11 +1577,7 @@ breakpoints_dbase_new (AnjutaPlugin *plugin)
 
 		/* Add popup menu */
 		bd->popup =  GTK_MENU (gtk_ui_manager_get_widget (GTK_UI_MANAGER (ui), "/PopupBreakpoint"));
-		if (bd->popup)
-		{	
-			gtk_widget_ref (GTK_WIDGET(bd->popup));
-			g_signal_connect (bd->treeview, "button-press-event", G_CALLBACK (on_breakpoints_button_press), bd);  
-		}
+		g_signal_connect (bd->treeview, "button-press-event", G_CALLBACK (on_breakpoints_button_press), bd);  
 			
 	}
 
@@ -1582,7 +1612,6 @@ breakpoints_dbase_destroy (BreakpointsDBase * bd)
 		g_free (bd->loc_history);
 
 	gtk_widget_destroy (bd->scrolledwindow);
-	if (bd->popup) gtk_widget_unref (GTK_WIDGET (bd->popup));
 	g_free (bd);
 }
 
