@@ -1,3 +1,4 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
     breakpoints.h
     Copyright (C) 2000  Naba Kumar <naba@gnome.org>
@@ -20,6 +21,8 @@
 #ifndef _BREAKPOINTS_DBASE_H_
 #define _BREAKPOINTS_DBASE_H_
 
+#include "plugin.h"
+
 #include <glade/glade.h>
 #include <stdio.h>
 #include <libanjuta/anjuta-plugin.h>
@@ -30,7 +33,7 @@ G_BEGIN_DECLS
 
 typedef struct _BreakpointsDBase BreakpointsDBase;
 
-BreakpointsDBase *breakpoints_dbase_new (AnjutaPlugin *plugin);
+BreakpointsDBase *breakpoints_dbase_new (DebugManagerPlugin *plugin);
 void breakpoints_dbase_destroy (BreakpointsDBase * bd);
 
 void breakpoints_dbase_connect (BreakpointsDBase *bd, IAnjutaDebugger *debugger);
