@@ -1,3 +1,4 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
     debugger.c
     Copyright (C) 2005 Sbastien Granjoux
@@ -97,10 +98,12 @@ typedef enum
 	HANDLE_SIGNAL_COMMAND,
 	LIST_LOCAL_COMMAND,
 	LIST_ARG_COMMAND,
+	LIST_THREAD_COMMAND,
+	SET_THREAD_COMMAND,
+	INFO_THREAD_COMMAND,
 	INFO_SIGNAL_COMMAND,    /* 0x20 */
 	INFO_FRAME_COMMAND,
 	INFO_ARGS_COMMAND,
-	INFO_THREADS_COMMAND,
 	INFO_VARIABLES_COMMAND,
 	SET_FRAME_COMMAND,
 	LIST_FRAME_COMMAND,
@@ -219,6 +222,15 @@ typedef enum
 	DMA_LIST_ARG_COMMAND =
 		LIST_ARG_COMMAND |
 		NEED_PROGRAM_STOPPED,
+	DMA_LIST_THREAD_COMMAND =
+		LIST_THREAD_COMMAND |
+		NEED_PROGRAM_STOPPED,
+	DMA_SET_THREAD_COMMAND =
+		SET_THREAD_COMMAND |
+		NEED_PROGRAM_STOPPED,
+	DMA_INFO_THREAD_COMMAND =
+		INFO_THREAD_COMMAND |
+		NEED_PROGRAM_STOPPED,
 	DMA_INFO_SIGNAL_COMMAND =
 		INFO_SIGNAL_COMMAND |
 		NEED_PROGRAM_STOPPED,
@@ -228,9 +240,6 @@ typedef enum
 	DMA_INFO_ARGS_COMMAND =
 		INFO_ARGS_COMMAND |
 		NEED_PROGRAM_STOPPED,
-	DMA_INFO_THREADS_COMMAND =
-		INFO_THREADS_COMMAND |
-		NEED_PROGRAM_STOPPED,
 	DMA_INFO_VARIABLES_COMMAND =
 		INFO_VARIABLES_COMMAND |
 		NEED_PROGRAM_STOPPED,
@@ -401,7 +410,7 @@ gboolean
 dma_queue_check_status (DmaDebuggerQueue *this, DmaDebuggerCommandType type, GError **err);
 
 void
-dma_queue_update_debugger_status (DmaDebuggerQueue *this, IAnjutaDebuggerStatus status);
+dma_queue_update_debugger_status (DmaDebuggerQueue *this, IAnjutaDebuggerStatus state, gint status);
 
 /* Message functions
  *---------------------------------------------------------------------------*/
@@ -509,6 +518,9 @@ dma_debugger_command_free (DmaQueueCommand *cmd)
 	    break;
 	case LIST_LOCAL_COMMAND:
 	case LIST_ARG_COMMAND:
+	case INFO_THREAD_COMMAND:
+	case LIST_THREAD_COMMAND:
+	case SET_THREAD_COMMAND:
 	case INFO_SIGNAL_COMMAND:
 	case INFO_SHAREDLIB_COMMAND:
 	case INFO_FRAME_COMMAND:
@@ -516,7 +528,6 @@ dma_debugger_command_free (DmaQueueCommand *cmd)
 	case INFO_TARGET_COMMAND:
 	case INFO_PROGRAM_COMMAND:
 	case INFO_UDOT_COMMAND:
-	case INFO_THREADS_COMMAND:
 	case INFO_VARIABLES_COMMAND:
 	case LIST_REGISTER_COMMAND:
 	case UPDATE_REGISTER_COMMAND:
@@ -866,12 +877,12 @@ dma_queue_emit_debugger_ready (DmaDebuggerQueue *queue)
 }
 
 void
-dma_queue_update_debugger_status (DmaDebuggerQueue *this, IAnjutaDebuggerStatus status)
+dma_queue_update_debugger_status (DmaDebuggerQueue *this, IAnjutaDebuggerStatus state, gint status)
 {
 	const char* signal = NULL;
 
 	this->queue_command = FALSE;
-	switch (status)
+	switch (state)
 	{
 	case IANJUTA_DEBUGGER_BUSY:
 		/* Debugger is busy, nothing to do */
@@ -966,7 +977,7 @@ dma_queue_update_debugger_status (DmaDebuggerQueue *this, IAnjutaDebuggerStatus
 	/* Emit signal */
 	if (signal != NULL)
 	{
-		g_signal_emit_by_name (this, signal);
+		g_signal_emit_by_name (this, signal, status);
 	}
 	this->queue_command = TRUE;
 }
@@ -1083,7 +1094,7 @@ dma_debugger_queue_execute (DmaDebuggerQueue *this)
 		IAnjutaDebuggerStatus status;
 		/* Recheck status in case of desynchronization */
 		status = ianjuta_debugger_get_status (this->debugger, NULL);
-		dma_queue_update_debugger_status (this, status);
+		dma_queue_update_debugger_status (this, status, 0);
 	}
 
 	/* Check if there is something to execute */
@@ -1121,9 +1132,7 @@ dma_debugger_queue_execute (DmaDebuggerQueue *this)
 		    ianjuta_debugger_unload (this->debugger, &err);
 		    break;
 		case QUIT_COMMAND:
-			DEBUG_PRINT ("quit command %p", err);
 			ret = ianjuta_debugger_quit (this->debugger, &err);
-			DEBUG_PRINT ("quit command %p ret %d", err, ret);
 			break;
 		case ABORT_COMMAND:
 			ianjuta_debugger_abort (this->debugger, &err);
@@ -1176,6 +1185,15 @@ dma_debugger_queue_execute (DmaDebuggerQueue *this)
 		case LIST_ARG_COMMAND:
 			ianjuta_debugger_list_argument (this->debugger, cmd->callback, cmd->user_data, &err);	
 			break;
+		case LIST_THREAD_COMMAND:
+			ianjuta_debugger_list_thread (this->debugger, cmd->callback, cmd->user_data, &err);	
+			break;
+		case SET_THREAD_COMMAND:
+			ianjuta_debugger_set_thread (this->debugger, cmd->frame.frame, &err);	
+			break;
+		case INFO_THREAD_COMMAND:
+			ianjuta_debugger_info_thread (this->debugger, cmd->info.id, cmd->callback, cmd->user_data, &err);	
+			break;
 		case INFO_SIGNAL_COMMAND:
 			ianjuta_debugger_info_signal (this->debugger, cmd->callback, cmd->user_data, &err);	
 			break;
@@ -1197,9 +1215,6 @@ dma_debugger_queue_execute (DmaDebuggerQueue *this)
 		case INFO_UDOT_COMMAND:
 			ianjuta_debugger_info_udot (this->debugger, cmd->callback, cmd->user_data, &err);	
 			break;
-		case INFO_THREADS_COMMAND:
-			ianjuta_debugger_info_threads (this->debugger, cmd->callback, cmd->user_data, &err);	
-			break;
 		case INFO_VARIABLES_COMMAND:
 			ianjuta_debugger_info_variables (this->debugger, cmd->callback, cmd->user_data, &err);	
 			break;
@@ -1328,7 +1343,7 @@ static void
 on_dma_debugger_ready (DmaDebuggerQueue *this, IAnjutaDebuggerStatus status)
 {
 	DEBUG_PRINT ("From debugger: receive debugger ready");
-	dma_queue_update_debugger_status (this, status);
+	dma_queue_update_debugger_status (this, status, 0);
 	if (this->ready) dma_debugger_queue_execute (this);
 }
 
@@ -1337,15 +1352,15 @@ on_dma_debugger_started (DmaDebuggerQueue *this)
 {
 	/* Nothing do to */
 	DEBUG_PRINT ("From debugger: receive debugger started");
-	dma_queue_update_debugger_status (this, IANJUTA_DEBUGGER_STARTED);
+	dma_queue_update_debugger_status (this, IANJUTA_DEBUGGER_STARTED, 0);
 	if (this->ready) dma_debugger_queue_execute (this);
 }
 
 static void
-on_dma_debugger_stopped (DmaDebuggerQueue *this)
+on_dma_debugger_stopped (DmaDebuggerQueue *this, gint status)
 {
 	DEBUG_PRINT ("From debugger: receive debugger stopped");
-	dma_queue_update_debugger_status (this, IANJUTA_DEBUGGER_STOPPED);
+	dma_queue_update_debugger_status (this, IANJUTA_DEBUGGER_STOPPED, status);
 	if (this->ready) dma_debugger_queue_execute (this);
 }
 
@@ -1353,7 +1368,7 @@ static void
 on_dma_program_loaded (DmaDebuggerQueue *this)
 {
 	DEBUG_PRINT ("From debugger: receive program loaded");
-	dma_queue_update_debugger_status (this, IANJUTA_DEBUGGER_PROGRAM_LOADED);
+	dma_queue_update_debugger_status (this, IANJUTA_DEBUGGER_PROGRAM_LOADED, 0);
 	if (this->ready) dma_debugger_queue_execute (this);
 }
 
@@ -1361,14 +1376,14 @@ static void
 on_dma_program_running (DmaDebuggerQueue *this)
 {
 	DEBUG_PRINT ("From debugger: debugger_program_running");
-	dma_queue_update_debugger_status (this, IANJUTA_DEBUGGER_PROGRAM_RUNNING);
+	dma_queue_update_debugger_status (this, IANJUTA_DEBUGGER_PROGRAM_RUNNING, 0);
 }
 
 static void
-on_dma_program_stopped (DmaDebuggerQueue *this)
+on_dma_program_stopped (DmaDebuggerQueue *this, guint thread)
 {
 	DEBUG_PRINT ("From debugger: receive program stopped");
-	dma_queue_update_debugger_status (this, IANJUTA_DEBUGGER_PROGRAM_STOPPED);
+	dma_queue_update_debugger_status (this, IANJUTA_DEBUGGER_PROGRAM_STOPPED, thread);
 	if (this->ready) dma_debugger_queue_execute (this);
 }
 
@@ -1376,7 +1391,7 @@ static void
 on_dma_program_exited (DmaDebuggerQueue *this)
 {
 	DEBUG_PRINT ("From debugger: receive program exited");
-	dma_queue_update_debugger_status (this, IANJUTA_DEBUGGER_PROGRAM_LOADED);
+	dma_queue_update_debugger_status (this, IANJUTA_DEBUGGER_PROGRAM_LOADED, 0);
 	if (this->ready) dma_debugger_queue_execute (this);
 }
 
@@ -1388,10 +1403,17 @@ on_dma_location_changed (DmaDebuggerQueue *this, const gchar* src_path, guint li
 }
 
 static void
-on_dma_frame_changed (DmaDebuggerQueue *this, guint frame)
+on_dma_frame_changed (DmaDebuggerQueue *this, guint frame, guint thread)
 {
 	DEBUG_PRINT ("From debugger: frame changed");
-	g_signal_emit_by_name (this, "frame-changed", frame);
+	g_signal_emit_by_name (this, "frame-changed", frame, thread);
+}
+
+static void
+on_dma_signal_received (DmaDebuggerQueue *this, const gchar* name, const gchar* description)
+{
+	DEBUG_PRINT ("From debugger: signal received");
+	g_signal_emit_by_name (this, "signal-received", name, description);
 }
 
 static void
@@ -1399,7 +1421,7 @@ on_dma_sharedlib_event (DmaDebuggerQueue *this)
 {
 	DEBUG_PRINT ("From debugger: shared lib event");
 	this->stop_on_sharedlib = TRUE;
-	dma_queue_update_debugger_status (this, IANJUTA_DEBUGGER_PROGRAM_STOPPED);
+	dma_queue_update_debugger_status (this, IANJUTA_DEBUGGER_PROGRAM_STOPPED, 0);
 	g_signal_emit_by_name (this, "sharedlib-event");
 	ianjuta_debugger_run (IANJUTA_DEBUGGER (this), NULL);
 }
@@ -1861,6 +1883,54 @@ idebugger_list_argument (IAnjutaDebugger *iface, IAnjutaDebuggerCallback callbac
 	return TRUE;
 }
 
+static gboolean
+idebugger_list_thread (IAnjutaDebugger *iface, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+{
+	DmaDebuggerQueue *this = DMA_DEBUGGER_QUEUE (iface);
+	DmaQueueCommand *cmd;
+	
+	cmd = dma_debugger_queue_append (this, DMA_LIST_THREAD_COMMAND, err);
+	if (cmd == NULL) return FALSE;
+		
+	cmd->callback = callback;
+	cmd->user_data = user_data;
+	dma_debugger_queue_execute (this);
+	
+	return TRUE;
+}
+
+static gboolean
+idebugger_set_thread (IAnjutaDebugger *iface, guint thread, GError **err)
+{
+	DmaDebuggerQueue *this = DMA_DEBUGGER_QUEUE (iface);
+	DmaQueueCommand *cmd;
+	
+	cmd = dma_debugger_queue_append (this, DMA_SET_THREAD_COMMAND, err);
+	if (cmd == NULL) return FALSE;
+		
+	cmd->frame.frame = thread;
+	dma_debugger_queue_execute (this);
+	
+	return TRUE;
+}
+
+static gboolean
+idebugger_info_thread (IAnjutaDebugger *iface, guint thread, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
+{
+	DmaDebuggerQueue *this = DMA_DEBUGGER_QUEUE (iface);
+	DmaQueueCommand *cmd;
+	
+	cmd = dma_debugger_queue_append (this, DMA_INFO_THREAD_COMMAND, err);
+	if (cmd == NULL) return FALSE;
+		
+	cmd->callback = callback;
+	cmd->user_data = user_data;
+	cmd->info.id = thread;
+	dma_debugger_queue_execute (this);
+	
+	return TRUE;
+}
+
 static gboolean
 idebugger_info_signal (IAnjutaDebugger *iface, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
 {
@@ -1992,22 +2062,6 @@ idebugger_info_udot (IAnjutaDebugger *iface, IAnjutaDebuggerCallback callback ,
 	return TRUE;
 }
 
-static gboolean
-idebugger_info_threads (IAnjutaDebugger *iface, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
-{
-	DmaDebuggerQueue *this = DMA_DEBUGGER_QUEUE (iface);
-	DmaQueueCommand *cmd;
-	
-	cmd = dma_debugger_queue_append (this, DMA_INFO_THREADS_COMMAND, err);
-	if (cmd == NULL) return FALSE;
-		
-	cmd->callback = callback;
-	cmd->user_data = user_data;
-	dma_debugger_queue_execute (this);
-	
-	return TRUE;
-}
-
 static gboolean
 idebugger_info_variables (IAnjutaDebugger *iface, IAnjutaDebuggerCallback callback , gpointer user_data, GError **err)
 {
@@ -2143,11 +2197,13 @@ idebugger_iface_init (IAnjutaDebuggerIface *iface)
 	iface->info_target = idebugger_info_target;
 	iface->info_program = idebugger_info_program;
 	iface->info_udot = idebugger_info_udot;
-	iface->info_threads = idebugger_info_threads;
 	iface->info_variables = idebugger_info_variables;
 	iface->handle_signal = idebugger_handle_signal;
 	iface->list_frame = idebugger_list_frame;
 	iface->set_frame = idebugger_set_frame;
+	iface->list_thread = idebugger_list_thread;
+	iface->set_thread = idebugger_set_thread;
+	iface->info_thread = idebugger_info_thread;
 	iface->list_register = idebugger_list_register;
 	iface->callback = idebugger_callback;
 
@@ -2518,6 +2574,7 @@ dma_debugger_queue_new (AnjutaPlugin *plugin)
 		g_signal_connect_swapped (this->debugger, "program-stopped", G_CALLBACK (on_dma_program_stopped), this);
 		g_signal_connect_swapped (this->debugger, "program-exited", G_CALLBACK (on_dma_program_exited), this);
 		g_signal_connect_swapped (this->debugger, "location-changed", G_CALLBACK (on_dma_location_changed), this);
+		g_signal_connect_swapped (this->debugger, "signal-received", G_CALLBACK (on_dma_signal_received), this);
 		g_signal_connect_swapped (this->debugger, "frame-changed", G_CALLBACK (on_dma_frame_changed), this);
 		g_signal_connect_swapped (this->debugger, "sharedlib-event", G_CALLBACK (on_dma_sharedlib_event), this);
 	}
