@@ -28,6 +28,7 @@
 #include "watch.h"
 #include "locals.h"
 #include "stack_trace.h"
+#include "threads.h"
 #include "info.h"
 #include "memory.h"
 #include "disassemble.h"
@@ -86,6 +87,7 @@ struct _DebugManagerPlugin
 	guint editor_watch_id;
 	IAnjutaEditor *pc_editor;
 	guint pc_line;
+	guint pc_address;
 	gboolean busy;
 	
 	/* Debugger components */
@@ -99,6 +101,7 @@ struct _DebugManagerPlugin
 	Signals *signals;	
 	DmaMemory *memory;
 	DmaDisassemble *disassemble;
+	DmaThreads *thread;
 	
 	
 	/* Logging view */
@@ -172,6 +175,13 @@ show_program_counter_in_editor(DebugManagerPlugin *self)
 	}
 }
 
+static void
+show_program_counter_in_disassembler(DebugManagerPlugin *self)
+{
+	dma_disassemble_mark (self->disassemble, self->pc_address, IANJUTA_MARKABLE_PROGRAM_COUNTER);
+	dma_disassemble_goto_address (self->disassemble, self->pc_address);
+}
+
 static void
 hide_program_counter_in_editor(DebugManagerPlugin *self)
 {
@@ -190,6 +200,12 @@ hide_program_counter_in_editor(DebugManagerPlugin *self)
 	}
 }
 
+static void
+hide_program_counter_in_disassembler(DebugManagerPlugin *self)
+{
+	dma_disassemble_clear_all_mark (self->disassemble, IANJUTA_MARKABLE_PROGRAM_COUNTER);
+}
+
 static void
 set_program_counter(DebugManagerPlugin *self, const gchar* file, guint line, guint address)
 {
@@ -198,7 +214,12 @@ set_program_counter(DebugManagerPlugin *self, const gchar* file, guint line, gui
 
 	/* Remove previous marker */
 	hide_program_counter_in_editor (self);
+	hide_program_counter_in_disassembler (self);
 	self->pc_editor = NULL;
+	self->pc_address = address;
+
+	if (address != 0)
+		show_program_counter_in_disassembler (self);
 
 	if (file != NULL)
 	{
@@ -484,7 +505,7 @@ dma_plugin_location_changed (DebugManagerPlugin *this, const gchar* file, guint
 {
 	gchar *msg;
 
-	DEBUG_PRINT ("DMA: dma_plugin_location_changed %s %d", file, line);
+	DEBUG_PRINT ("DMA: dma_plugin_location_changed %s %d %x", file, line, address);
 	
 	msg = g_strdup_printf (_("Location: %s, line %d\n"), file, line);
 	dma_debugger_message (this->queue, msg);
@@ -534,38 +555,58 @@ dma_plugin_program_unload (DebugManagerPlugin *this)
 /* Called when the debugger is stopped */
 
 static void
-dma_plugin_debugger_stopped (DebugManagerPlugin *this)
+dma_plugin_debugger_stopped (DebugManagerPlugin *self, gint status)
 {
 	AnjutaUI *ui;
 	GtkAction *action;
-	AnjutaStatus* status;
+	AnjutaStatus* state;
 
 	DEBUG_PRINT ("DMA: dma_plugin_debugger_stopped");
 
-	dma_plugin_program_unload (this);
+	dma_plugin_program_unload (self);
 	
 	/* Update ui */
-	ui = anjuta_shell_get_ui (ANJUTA_PLUGIN (this)->shell, NULL);
-	gtk_action_group_set_visible (this->start_group, TRUE);
-	gtk_action_group_set_sensitive (this->start_group, TRUE);
-	action = gtk_action_group_get_action (this->start_group, "ActionDebuggerStop");
+	ui = anjuta_shell_get_ui (ANJUTA_PLUGIN (self)->shell, NULL);
+	gtk_action_group_set_visible (self->start_group, TRUE);
+	gtk_action_group_set_sensitive (self->start_group, TRUE);
+	action = gtk_action_group_get_action (self->start_group, "ActionDebuggerStop");
 	gtk_action_set_sensitive (action, FALSE);
-	gtk_action_group_set_visible (this->loaded_group, TRUE);
-	gtk_action_group_set_sensitive (this->loaded_group, FALSE);
-	gtk_action_group_set_visible (this->stopped_group, TRUE);
-	gtk_action_group_set_sensitive (this->stopped_group, FALSE);
-	gtk_action_group_set_visible (this->running_group, TRUE);
-	gtk_action_group_set_sensitive (this->running_group, FALSE);
+	gtk_action_group_set_visible (self->loaded_group, TRUE);
+	gtk_action_group_set_sensitive (self->loaded_group, FALSE);
+	gtk_action_group_set_visible (self->stopped_group, TRUE);
+	gtk_action_group_set_sensitive (self->stopped_group, FALSE);
+	gtk_action_group_set_visible (self->running_group, TRUE);
+	gtk_action_group_set_sensitive (self->running_group, FALSE);
 
 	/* clear indicator */
-	set_program_counter (this, NULL, 0, 0);
+	set_program_counter (self, NULL, 0, 0);
 	
-	enable_log_view (this, FALSE);
+	enable_log_view (self, FALSE);
 	
-	status = anjuta_shell_get_status(ANJUTA_PLUGIN (this)->shell, NULL);
-	anjuta_status_set_default (status, _("Debugger"), NULL);
+	state = anjuta_shell_get_status(ANJUTA_PLUGIN (self)->shell, NULL);
+	anjuta_status_set_default (state, _("Debugger"), NULL);
 	
-	dma_plugin_debugger_ready (this, IANJUTA_DEBUGGER_STOPPED);
+	dma_plugin_debugger_ready (self, IANJUTA_DEBUGGER_STOPPED);
+	
+	/* Display a warning if debugger stop unexpectedly */
+	if (status != 0)
+	{
+		GtkWindow *parent = GTK_WINDOW (ANJUTA_PLUGIN(self)->shell);
+		anjuta_util_dialog_error (parent, _("gdb terminated unexpectedly with status %d\n"), status);
+	}
+		
+}
+
+static void
+dma_plugin_signal_received (DebugManagerPlugin *self, const gchar *name, const gchar *description)
+{
+	GtkWindow *parent = GTK_WINDOW (ANJUTA_PLUGIN (self)->shell);
+	
+	/* Skip SIGINT signal */
+	if (strcmp(name, "SIGINT") != 0)
+	{
+		anjuta_util_dialog_warning (parent, _("Program has received signal: %s\n"), description);
+	}
 }
 
 /* Start/Stop menu functions
@@ -745,12 +786,6 @@ on_info_udot_activate (GtkAction *action, DebugManagerPlugin *plugin)
 	ianjuta_debugger_info_udot (plugin->debugger, on_debugger_dialog_message, plugin, NULL);
 }
 
-static void
-on_info_threads_activate (GtkAction *action, DebugManagerPlugin *plugin)
-{
-	ianjuta_debugger_info_threads (plugin->debugger, on_debugger_dialog_message, plugin, NULL);
-}
-
 static void
 on_info_variables_activate (GtkAction *action, DebugManagerPlugin *plugin)
 {
@@ -982,14 +1017,6 @@ static GtkActionEntry actions_stopped[] =
 		NULL,
 		NULL
 	},
-	{
-		"ActionGdbInfoThreads",
-		NULL,
-		N_("Info _Threads"),
-		NULL,
-		N_("Display the IDs of currently known threads"),
-		G_CALLBACK (on_info_threads_activate)
-	},
 	{
 		"ActionGdbInfoGlobalVariables",
 		NULL,
@@ -1102,6 +1129,7 @@ dma_plugin_activate (AnjutaPlugin* plugin)
 	g_signal_connect_swapped (this->debugger, "program-stopped", G_CALLBACK (dma_plugin_program_stopped), this);
 	g_signal_connect_swapped (this->debugger, "program-exited", G_CALLBACK (dma_plugin_program_loaded), this);
 	g_signal_connect_swapped (this->debugger, "location-changed", G_CALLBACK (dma_plugin_location_changed), this);
+	g_signal_connect_swapped (this->debugger, "signal-received", G_CALLBACK (dma_plugin_signal_received), this);
 	g_signal_connect (this->debugger, "debugger-ready", G_CALLBACK (on_debugger_ready_signal), this);
 
 	/* Watch expression */
@@ -1111,10 +1139,13 @@ dma_plugin_activate (AnjutaPlugin* plugin)
 	this->locals = locals_new (ANJUTA_PLUGIN (plugin), this->debugger);
 
 	/* Stack trace */
-	this->stack = stack_trace_new (this->debugger, ANJUTA_PLUGIN (this));
+	this->stack = stack_trace_new (this->debugger, this);
 
+	/* Thread list */
+	this->thread = dma_threads_new (this->debugger, this);
+	
 	/* Create breakpoints list */
-	this->breakpoints = breakpoints_dbase_new (plugin);
+	this->breakpoints = breakpoints_dbase_new (this);
 
 	/* Register list */
 	this->registers = cpu_registers_new (plugin, this->debugger);
@@ -1124,12 +1155,12 @@ dma_plugin_activate (AnjutaPlugin* plugin)
 
 	/* Disassembly window */
 	this->disassemble = dma_disassemble_new (plugin, this->debugger);
-	
+
 	/* Start debugger part */
 	this->start = dma_start_new (plugin, this->debugger);
 	
 
-	dma_plugin_debugger_stopped (this);
+	dma_plugin_debugger_stopped (this, 0);
 	action = gtk_action_group_get_action (this->start_group, "ActionDebuggerRestartTarget");
 	gtk_action_set_sensitive (action, FALSE);
 	
@@ -1161,7 +1192,7 @@ dma_plugin_deactivate (AnjutaPlugin* plugin)
 	this = ANJUTA_PLUGIN_DEBUG_MANAGER (plugin);
 
 	/* Stop debugger */
-	dma_plugin_debugger_stopped (this);
+	dma_plugin_debugger_stopped (this, 0);
 	
 	g_signal_handlers_disconnect_by_func (this->queue, G_CALLBACK (dma_plugin_debugger_started), this);
 	g_signal_handlers_disconnect_by_func (this->queue, G_CALLBACK (dma_plugin_debugger_stopped), this);
@@ -1197,6 +1228,9 @@ dma_plugin_deactivate (AnjutaPlugin* plugin)
 	dma_disassemble_free (this->disassemble);
 	this->disassemble = NULL;
 	
+	dma_threads_free (this->thread);
+	this->thread = NULL;
+	
 	dma_start_free (this->start);
 	this->start = NULL;
 
@@ -1215,6 +1249,25 @@ dma_plugin_deactivate (AnjutaPlugin* plugin)
 	return TRUE;
 }
 
+/* Public functions
+ *---------------------------------------------------------------------------*/
+
+void
+dma_debug_manager_goto_code (DebugManagerPlugin *self, const gchar *uri, guint line, guint address)
+{
+	/* Go to address in disassembly view */
+	if (address != 0)
+	{
+		dma_disassemble_goto_address (self->disassemble, address);	
+	}
+
+	/* Go to location in editor */
+	if (uri != NULL)
+	{
+		goto_location_in_editor (ANJUTA_PLUGIN (self), uri, line);
+	}
+}
+
 
 /* GObject functions
  *---------------------------------------------------------------------------*/
