@@ -52,14 +52,15 @@
 #define ICON_FILE "anjuta-document-manager.png"
 
 /* Pixmaps */
-#define ANJUTA_PIXMAP_SWAP                "undock.png"
+#define ANJUTA_PIXMAP_SWAP                "swap.png"
 #define ANJUTA_PIXMAP_INDENT              "indent.xpm"
 #define ANJUTA_PIXMAP_SYNTAX              "syntax.xpm"
-#define ANJUTA_PIXMAP_BOOKMARK_TOGGLE     "bookmark_toggle.xpm"
+#define ANJUTA_PIXMAP_BOOKMARK_TOGGLE     "bookmark-toggle.png"
 #define ANJUTA_PIXMAP_BOOKMARK_FIRST      "bookmark-first.png"
 #define ANJUTA_PIXMAP_BOOKMARK_PREV       "bookmark-prev.png"
 #define ANJUTA_PIXMAP_BOOKMARK_NEXT       "bookmark-next.png"
 #define ANJUTA_PIXMAP_BOOKMARK_LAST       "bookmark-last.png"
+#define ANJUTA_PIXMAP_BOOKMARK_CLEAR      "bookmark-clear.png"
 #define ANJUTA_PIXMAP_ERROR_PREV          "error-prev.png"
 #define ANJUTA_PIXMAP_ERROR_NEXT          "error-next.png"
 
@@ -99,7 +100,7 @@
 #define ANJUTA_STOCK_BOOKMARK_PREV            "anjuta-bookmark-previous"
 #define ANJUTA_STOCK_BOOKMARK_NEXT            "anjuta-bookmark-next"
 #define ANJUTA_STOCK_BOOKMARK_LAST            "anjuta-bookmark-last"
-
+#define ANJUTA_STOCK_BOOKMARK_CLEAR           "anjuta-bookmark-clear"
 
 static gpointer parent_class;
 
@@ -386,7 +387,7 @@ static GtkActionEntry actions_bookmark[] = {
 	N_("Jump to the last bookmark in the file"),
     G_CALLBACK (on_editor_command_bookmark_last_activate)},
   { "ActionBookmarkClear", N_("_Clear All Bookmarks"),
-	NULL, NULL,
+	ANJUTA_STOCK_BOOKMARK_CLEAR, NULL,
 	N_("Clear bookmarks"),
     G_CALLBACK (on_editor_command_bookmark_clear_activate)},
 };
@@ -719,6 +720,8 @@ update_editor_ui_interface_items (AnjutaPlugin *plugin, IAnjutaEditor *editor)
 	g_object_set(G_OBJECT (action), "visible", flag, NULL);
 	action = anjuta_ui_get_action(ui,  "ActionGroupEditorNavigate", "ActionEditGotoBlockEnd");
 	g_object_set(G_OBJECT (action), "visible", flag, NULL);
+	action = anjuta_ui_get_action(ui,  "ActionGroupEditorNavigate", "ActionEditGotoMatchingBrace");
+	g_object_set(G_OBJECT (action), "visible", flag, NULL);	
 }
 
 static void
@@ -779,6 +782,7 @@ register_stock_icons (AnjutaPlugin *plugin)
 	REGISTER_ICON (ANJUTA_PIXMAP_BOOKMARK_PREV, ANJUTA_STOCK_BOOKMARK_PREV);
 	REGISTER_ICON (ANJUTA_PIXMAP_BOOKMARK_NEXT, ANJUTA_STOCK_BOOKMARK_NEXT);
 	REGISTER_ICON (ANJUTA_PIXMAP_BOOKMARK_LAST, ANJUTA_STOCK_BOOKMARK_LAST);
+	REGISTER_ICON (ANJUTA_PIXMAP_BOOKMARK_CLEAR, ANJUTA_STOCK_BOOKMARK_CLEAR);
 	REGISTER_ICON (ANJUTA_PIXMAP_BLOCK_START, ANJUTA_STOCK_BLOCK_START);
 	REGISTER_ICON (ANJUTA_PIXMAP_BLOCK_END, ANJUTA_STOCK_BLOCK_END);
 }
@@ -1192,16 +1196,20 @@ on_session_save (AnjutaShell *shell, AnjutaSessionPhase phase,
 	while (node)
 	{
 		IAnjutaEditor *te;
+		gchar *te_uri;
+		
 		te = IANJUTA_EDITOR (node->data);
-		if (ianjuta_file_get_uri(IANJUTA_FILE(te), NULL))
+		te_uri = ianjuta_file_get_uri(IANJUTA_FILE(te), NULL);
+		if (te_uri)
 		{
 			gchar *uri;
 			/* Save line locations also */
-			uri = g_strdup_printf ("%s#%d", ianjuta_file_get_uri(IANJUTA_FILE(te), NULL),
+			uri = g_strdup_printf ("%s#%d", te_uri,
 								  ianjuta_editor_get_lineno(IANJUTA_EDITOR(te), NULL));
 			files = g_list_prepend (files, uri);
 			/* uri not be freed here */
 		}
+		g_free (te_uri);
 		node = g_list_next (node);
 	}
 	files = g_list_reverse (files);
@@ -1237,12 +1245,13 @@ on_save_prompt (AnjutaShell *shell, AnjutaSavePrompt *save_prompt,
 		if (ianjuta_file_savable_is_dirty (editor, NULL))
 		{
 			const gchar *name;
-			const gchar *uri;
+			gchar *uri;
 			
 			name = ianjuta_editor_get_filename (IANJUTA_EDITOR (editor), NULL);
 			uri = ianjuta_file_get_uri (IANJUTA_FILE (editor), NULL);
 			anjuta_save_prompt_add_item (save_prompt, name, uri, editor,
 										 on_save_prompt_save_editor, plugin);
+			g_free (uri);
 		}
 		node = g_list_next (node);
 	}
@@ -1317,9 +1326,11 @@ on_docman_auto_save (gpointer data)
 		editor = IANJUTA_EDITOR(editors->data);
 		if (ianjuta_file_savable_is_dirty(IANJUTA_FILE_SAVABLE(editor), NULL))
 		{
-			if (ianjuta_file_get_uri(IANJUTA_FILE(editor), NULL) != NULL)
+			gchar *uri = ianjuta_file_get_uri(IANJUTA_FILE(editor), NULL);
+			if (uri != NULL)
 			{
 				ianjuta_file_savable_save(IANJUTA_FILE_SAVABLE(editor), NULL);
+				g_free (uri);
 			}
 		}
 		editors = g_list_next(editors);
@@ -1684,7 +1695,7 @@ ianjuta_docman_goto_file_line (IAnjutaDocumentManager *plugin,
 {
 	AnjutaDocman *docman;
 	docman = ANJUTA_DOCMAN ((ANJUTA_PLUGIN_DOCMAN (plugin)->docman));
-	anjuta_docman_goto_file_line (docman, uri, linenum);
+	return anjuta_docman_goto_file_line (docman, uri, linenum);
 }
 
 static IAnjutaEditor*
@@ -1693,7 +1704,7 @@ ianjuta_docman_goto_file_line_mark (IAnjutaDocumentManager *plugin,
 {
 	AnjutaDocman *docman;
 	docman = ANJUTA_DOCMAN ((ANJUTA_PLUGIN_DOCMAN (plugin)->docman));
-	anjuta_docman_goto_file_line_mark (docman, uri, linenum, mark);
+	return anjuta_docman_goto_file_line_mark (docman, uri, linenum, mark);
 }
 
 static IAnjutaEditor*
@@ -1767,7 +1778,7 @@ ifile_get_uri (IAnjutaFile* plugin, GError** e)
 	docman = ANJUTA_DOCMAN ((ANJUTA_PLUGIN_DOCMAN (plugin)->docman));
 	editor = anjuta_docman_get_current_editor (docman);
 	if (editor != NULL)
-		return g_strdup (ianjuta_file_get_uri(IANJUTA_FILE(editor), NULL));
+		return ianjuta_file_get_uri(IANJUTA_FILE(editor), NULL);
 	else if (ianjuta_editor_get_filename(editor, NULL))
 		return gnome_vfs_get_uri_from_local_path (ianjuta_editor_get_filename(editor, NULL));
 	else
@@ -1793,12 +1804,15 @@ isaveable_save (IAnjutaFileSavable* plugin, GError** e)
 	editors = anjuta_docman_get_all_editors(docman);
 	while(editors)
 	{
+		gchar *uri;
 		editor = IANJUTA_EDITOR(editors->data);
-		if (ianjuta_file_get_uri(IANJUTA_FILE(editor), NULL) != NULL &&
+		uri = ianjuta_file_get_uri(IANJUTA_FILE(editor), NULL);
+		if (uri != NULL &&
 			ianjuta_file_savable_is_dirty(IANJUTA_FILE_SAVABLE(editor), NULL))
 		{
 			ianjuta_file_savable_save(IANJUTA_FILE_SAVABLE(editor), NULL);
 		}
+		g_free (uri);
 		editors = g_list_next(editors);
 	}
 	g_list_free(editors);
